@@ -40,7 +40,6 @@ class P4Source(base.ChangeSource, util.ComparableMixin):
     parent = None # filled in when we're added
     last_change = None
     loop = None
-    volatile = ['loop']
     working = False
 
     def __init__(self, p4port=None, p4user=None, p4passwd=None,
@@ -75,9 +74,9 @@ class P4Source(base.ChangeSource, util.ComparableMixin):
         self.split_file = split_file
         self.pollinterval = pollinterval
         self.histmax = histmax
+        self.loop = LoopingCall(self.checkp4)
 
     def startService(self):
-        self.loop = LoopingCall(self.checkp4)
         base.ChangeSource.startService(self)
 
         # Don't start the loop just yet because the reactor isn't running.
@@ -166,6 +165,9 @@ class P4Source(base.ChangeSource, util.ComparableMixin):
 
     def _process_describe(self, result, num):
         lines = result.split('\n')
+        # SF#1555985: Wade Brainerd reports a stray ^M at the end of the date
+        # field. The rstrip() is intended to remove that.
+        lines[0] = lines[0].rstrip()
         m = self.describe_header_re.match(lines[0])
         assert m, "Unexpected 'p4 describe -s' result: %r" % result
         who = m.group('who')
