@@ -98,7 +98,7 @@ class IStatus(Interface):
     def getBuildbotURL():
         """Return the URL of the top-most Buildbot status page, or None if
         this Buildbot does not provide a web status page."""
-    def getURLFor(thing):
+    def getURLForThing(thing):
         """Return the URL of a page which provides information on 'thing',
         which should be an object that implements one of the status
         interfaces defined in L{buildbot.interfaces}. Returns None if no
@@ -557,6 +557,11 @@ class IStatusEvent(Interface):
         """Returns a single string with the color that should be used to
         display this event. 'red' and 'yellow' are the most likely ones."""
 
+
+LOG_CHANNEL_STDOUT = 0
+LOG_CHANNEL_STDERR = 1
+LOG_CHANNEL_HEADER = 2
+
 class IStatusLog(Interface):
     """I represent a single Log, which is a growing list of text items that
     contains some kind of output for a single BuildStep. I might be finished,
@@ -661,6 +666,12 @@ class IStatusLog(Interface):
         """Return one big string with the contents of the Log. This merges
         all non-header chunks together."""
 
+    def readlines(channel=LOG_CHANNEL_STDOUT):
+        """Read lines from one channel of the logfile. This returns an
+        iterator that will provide single lines of text (including the
+        trailing newline).
+        """
+
     def getTextWithHeaders():
         """Return one big string with the contents of the Log. This merges
         all chunks (including headers) together."""
@@ -670,10 +681,6 @@ class IStatusLog(Interface):
         0 for stdout, 1 for stderr, 2 for header. (note that stderr is merged
         into stdout if PTYs are in use)."""
 
-LOG_CHANNEL_STDOUT = 0
-LOG_CHANNEL_STDERR = 1
-LOG_CHANNEL_HEADER = 2
-
 class IStatusLogConsumer(Interface):
     """I am an object which can be passed to IStatusLog.subscribeConsumer().
     I represent a target for writing the contents of an IStatusLog. This
@@ -819,26 +826,6 @@ class IControl(Interface):
         """Retrieve the IBuilderControl object for the given Builder."""
 
 class IBuilderControl(Interface):
-    def forceBuild(who, reason):
-        """DEPRECATED, please use L{requestBuild} instead.
-
-        Start a build of the latest sources. If 'who' is not None, it is
-        string with the name of the user who is responsible for starting the
-        build: they will be added to the 'interested users' list (so they may
-        be notified via email or another Status object when it finishes).
-        'reason' is a string describing why this user requested the build.
-
-        The results of forced builds are always sent to the Interested Users,
-        even if the Status object would normally only send results upon
-        failures.
-
-        forceBuild() may raise L{NoSlaveError} or L{BuilderInUseError} if it
-        cannot start the build.
-
-        forceBuild() returns a Deferred which fires with an L{IBuildControl}
-        object that can be used to further control the new build, or from
-        which an L{IBuildStatus} object can be obtained."""
-
     def requestBuild(request):
         """Queue a L{buildbot.process.base.BuildRequest} object for later
         building."""
