@@ -1,16 +1,13 @@
 # -*- test-case-name: buildbot.test.test_step -*-
 
-import types, time
-from StringIO import StringIO
+import types
 
-from twisted.python import log, components
+from twisted.python import log
 from twisted.python.failure import Failure
 from twisted.internet import reactor, defer, error
-from twisted.spread import pb
 
 from buildbot import interfaces
 from buildbot.twcompat import implements
-from buildbot.util import now
 from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, EXCEPTION
 from buildbot.status.builder import Results, BuildRequestStatus
 from buildbot.status.progress import BuildProgress
@@ -276,6 +273,7 @@ class Build:
     def setupSlaveBuilder(self, slavebuilder):
         self.slavebuilder = slavebuilder
         self.slavename = slavebuilder.slave.slavename
+        self.build_status.setSlavename(self.slavename)
         self.setProperty("slavename", self.slavename)
 
     def startBuild(self, build_status, expectations, slavebuilder):
@@ -301,6 +299,10 @@ class Build:
         self.remote = slavebuilder.remote
         self.remote.notifyOnDisconnect(self.lostRemote)
         d = self.deferred = defer.Deferred()
+        def _release_slave(res):
+            self.slavebuilder.buildFinished()
+            return res
+        d.addCallback(_release_slave)
 
         try:
             self.setupBuild(expectations) # create .steps
@@ -332,7 +334,7 @@ class Build:
         for lock in self.locks:
             if not lock.isAvailable():
                 log.msg("Build %s waiting for lock %s" % (self, lock))
-                d = lock.waitUntilAvailable(self)
+                d = lock.waitUntilMaybeAvailable(self)
                 d.addCallback(self.acquireLocks)
                 return d
         # all locks are available, claim them all
@@ -546,7 +548,6 @@ class Build:
         self.results = results
 
         log.msg(" %s: build finished" % self)
-        self.build_status.setSlavename(self.slavename)
         self.build_status.setText(text)
         self.build_status.setColor(color)
         self.build_status.setResults(results)
