@@ -10,7 +10,6 @@ from twisted.cred import portal, checkers
 from twisted.spread import pb
 
 from buildbot import interfaces, buildset, util, pbutil
-from buildbot.util import now
 from buildbot.status import builder
 from buildbot.twcompat import implements, providedBy
 from buildbot.sourcestamp import SourceStamp
@@ -107,8 +106,12 @@ class Scheduler(BaseUpstreamScheduler):
 
         BaseUpstreamScheduler.__init__(self, name)
         self.treeStableTimer = treeStableTimer
+        errmsg = ("The builderNames= argument to Scheduler must be a list "
+                  "of Builder description names (i.e. the 'name' key of the "
+                  "Builder specification dictionary)")
+        assert isinstance(builderNames, (list, tuple)), errmsg
         for b in builderNames:
-            assert isinstance(b, str)
+            assert isinstance(b, str), errmsg
         self.builderNames = builderNames
         self.branch = branch
         if fileIsImportant:
@@ -331,6 +334,8 @@ class Periodic(BaseUpstreamScheduler):
         self.builderNames = builderNames
         self.periodicBuildTimer = periodicBuildTimer
         self.branch = branch
+        self.reason = ("The Periodic scheduler named '%s' triggered this build"
+                       % name)
         self.timer = internet.TimerService(self.periodicBuildTimer,
                                            self.doPeriodicBuild)
         self.timer.setServiceParent(self)
@@ -345,7 +350,8 @@ class Periodic(BaseUpstreamScheduler):
 
     def doPeriodicBuild(self):
         bs = buildset.BuildSet(self.builderNames,
-                               SourceStamp(branch=self.branch))
+                               SourceStamp(branch=self.branch),
+                               self.reason)
         self.submit(bs)
 
 
@@ -383,7 +389,9 @@ class Nightly(BaseUpstreamScheduler):
                  month=12, dayOfMonth=24, hour=12, minute=0)
 
     For dayOfWeek and dayOfMonth, builds are triggered if the date matches
-    either of them.  Month and day numbers start at 1, not zero.
+    either of them. All time values are compared against the tuple returned
+    by time.localtime(), so month and dayOfMonth numbers start at 1, not
+    zero. dayOfWeek=0 is Monday, dayOfWeek=6 is Sunday.
     """
 
     compare_attrs = ('name', 'builderNames',
@@ -406,6 +414,8 @@ class Nightly(BaseUpstreamScheduler):
         self.branch = branch
         self.delayedRun = None
         self.nextRunTime = None
+        self.reason = ("The Nightly scheduler named '%s' triggered this build"
+                       % name)
 
     def addTime(self, timetuple, secs):
         return time.localtime(time.mktime(timetuple)+secs)
@@ -498,7 +508,8 @@ class Nightly(BaseUpstreamScheduler):
 
         # And trigger a build
         bs = buildset.BuildSet(self.builderNames,
-                               SourceStamp(branch=self.branch))
+                               SourceStamp(branch=self.branch),
+                               self.reason)
         self.submit(bs)
 
     def addChange(self, change):
