@@ -1,9 +1,9 @@
 #! /usr/bin/python
 
-import time, os, os.path, re, sys
+import os.path
 
 from twisted.spread import pb
-from twisted.python import log, usage, failure
+from twisted.python import log
 from twisted.internet import reactor, defer
 from twisted.application import service, internet
 from twisted.cred import credentials
@@ -65,9 +65,9 @@ class SlaveBuilder(pb.Referenceable, service.Service):
         #service.Service.__init__(self) # Service has no __init__ method
         self.setName(name)
         self.not_really = not_really
-        
+
     def __repr__(self):
-        return "<SlaveBuilder '%s'>" % self.name
+        return "<SlaveBuilder '%s' at %d>" % (self.name, id(self))
 
     def setServiceParent(self, parent):
         service.Service.setServiceParent(self, parent)
@@ -284,7 +284,9 @@ class Bot(pb.Referenceable, service.MultiService):
 
     def remote_setBuilderList(self, wanted):
         retval = {}
+        wanted_dirs = []
         for (name, builddir) in wanted:
+            wanted_dirs.append(builddir)
             b = self.builders.get(name, None)
             if b:
                 if b.builddir != builddir:
@@ -303,8 +305,15 @@ class Bot(pb.Referenceable, service.MultiService):
                 log.msg("removing old builder %s" % name)
                 self.builders[name].disownServiceParent()
                 del(self.builders[name])
+
+        for d in os.listdir(self.basedir):
+            if os.path.isdir(d):
+                if d not in wanted_dirs:
+                    log.msg("I have a leftover directory '%s' that is not "
+                            "being used by the buildmaster: you can delete "
+                            "it now" % d)
         return retval
-    
+
     def remote_print(self, message):
         log.msg("message from master:", message)
 
@@ -326,10 +335,6 @@ class Bot(pb.Referenceable, service.MultiService):
                 files[f] = open(filename, "r").read()
         return files
 
-    def debug_forceBuild(self, name):
-        d = self.perspective.callRemote("forceBuild", name)
-        d.addCallbacks(log.msg, log.err)
-
 class BotFactory(ReconnectingPBClientFactory):
     # 'keepaliveInterval' serves two purposes. The first is to keep the
     # connection alive: it guarantees that there will be at least some
@@ -457,6 +462,7 @@ class BuildSlave(service.MultiService):
 
     def __init__(self, host, port, name, passwd, basedir, keepalive,
                  usePTY, keepaliveTimeout=30, umask=None, debugOpts={}):
+        log.msg("Creating BuildSlave")
         service.MultiService.__init__(self)
         self.debugOpts = debugOpts.copy()
         bot = self.botClass(basedir, usePTY)
