@@ -1,6 +1,6 @@
 # -*- test-case-name: buildbot.test.test_slavecommand -*-
 
-import os, os.path, re, signal, shutil, types, time
+import os, re, signal, shutil, types, time
 from stat import ST_CTIME, ST_MTIME, ST_SIZE
 
 from twisted.internet.protocol import ProcessProtocol
@@ -14,7 +14,7 @@ from buildbot.slave.registry import registerSlaveCommand
 # this used to be a CVS $-style "Revision" auto-updated keyword, but since I
 # moved to Darcs as the primary repository, this is updated manually each
 # time this file is changed. The last cvs_ver that was here was 1.51 .
-command_version = "2.1"
+command_version = "2.2"
 
 # version history:
 #  >=1.17: commands are interruptable
@@ -32,6 +32,7 @@ command_version = "2.1"
 #          ShellCommand accepts new arguments (logfiles=, initialStdin=,
 #          keepStdinOpen=) and no longer accepts stdin=)
 #          (release 0.7.4)
+#  >= 2.2: added monotone, uploadFile, and downloadFile (release 0.7.5)
 
 class CommandInterrupted(Exception):
     pass
@@ -173,6 +174,8 @@ class LogFileWatcher:
         self.poll()
         if self.poller is not None:
             self.poller.stop()
+        if self.started:
+            self.f.close()
 
     def statFile(self):
         if os.path.exists(self.logfile):
@@ -193,6 +196,7 @@ class LogFileWatcher:
                 return # no file to work with
             self.f = open(self.logfile, "rb")
             self.started = True
+        self.f.seek(self.f.tell(), 0)
         while True:
             data = self.f.read(10000)
             if not data:
@@ -666,6 +670,244 @@ class Command:
         return None
 
 
+
+class SlaveFileUploadCommand(Command):
+    """
+    Upload a file from slave to build master
+    Arguments:
+
+        - ['workdir']:   base directory to use
+        - ['slavesrc']:  name of the slave-side file to read from
+        - ['writer']:    RemoteReference to a transfer._FileWriter object
+        - ['maxsize']:   max size (in bytes) of file to write
+        - ['blocksize']: max size for each data block
+    """
+    debug = False
+
+    def setup(self, args):
+        self.workdir = args['workdir']
+        self.filename = args['slavesrc']
+        self.writer = args['writer']
+        self.remaining = args['maxsize']
+        self.blocksize = args['blocksize']
+        self.stderr = None
+        self.rc = 0
+
+    def start(self):
+	if self.debug:
+            log.msg('SlaveFileUploadCommand started')
+
+        # Open file
+        self.path = os.path.join(self.builder.basedir,
+                                 self.workdir,
+                                 os.path.expanduser(self.filename))
+        try:
+            self.fp = open(self.path, 'r')
+            if self.debug:
+                log.msg('Opened %r for upload' % self.path)
+        except:
+            # TODO: this needs cleanup
+            self.fp = None
+            self.stderr = 'Cannot open file %r for upload' % self.path
+            self.rc = 1
+            if self.debug:
+                log.msg('Cannot open file %r for upload' % self.path)
+
+        self.sendStatus({'header': "sending %s" % self.path})
+
+        d = defer.Deferred()
+        d.addCallback(self._writeBlock)
+        d.addBoth(self.finished)
+        reactor.callLater(0, d.callback, None)
+        return d
+
+    def _writeBlock(self, res):
+        """
+        Write a block of data to the remote writer
+        """
+        if self.interrupted or self.fp is None:
+            if self.debug:
+                log.msg('SlaveFileUploadCommand._writeBlock(): end')
+            d = self.writer.callRemote('close')
+            return d
+
+        length = self.blocksize
+        if self.remaining is not None and length > self.remaining:
+            length = self.remaining
+
+        if length <= 0:
+            if self.stderr is None:
+                self.stderr = 'Maximum filesize reached, truncating file %r' \
+                                % self.path
+                self.rc = 1
+            data = ''
+        else:
+            data = self.fp.read(length)
+
+        if self.debug:
+            log.msg('SlaveFileUploadCommand._writeBlock(): '+
+                    'allowed=%d readlen=%d' % (length, len(data)))
+        if len(data) == 0:
+            d = self.writer.callRemote('close')
+            return d
+
+        if self.remaining is not None:
+            self.remaining = self.remaining - len(data)
+            assert self.remaining >= 0
+        d = self.writer.callRemote('write', data)
+        d.addCallback(self._writeBlock)
+        return d
+
+    def interrupt(self):
+        if self.debug:
+            log.msg('interrupted')
+        if self.interrupted:
+            return
+        if self.stderr is None:
+            self.stderr = 'Upload of %r interrupted' % self.path
+            self.rc = 1
+        self.interrupted = True
+        # the next _writeBlock call will notice the .interrupted flag
+
+    def finished(self, res):
+        if self.debug:
+            log.msg('finished: stderr=%r, rc=%r' % (self.stderr, self.rc))
+        if self.stderr is None:
+            self.sendStatus({'rc': self.rc})
+        else:
+            self.sendStatus({'stderr': self.stderr, 'rc': self.rc})
+        return res
+
+registerSlaveCommand("uploadFile", SlaveFileUploadCommand, command_version)
+
+
+class SlaveFileDownloadCommand(Command):
+    """
+    Download a file from master to slave
+    Arguments:
+
+        - ['workdir']:   base directory to use
+        - ['slavedest']: name of the slave-side file to be created
+        - ['reader']:    RemoteReference to a transfer._FileReader object
+        - ['maxsize']:   max size (in bytes) of file to write
+        - ['blocksize']: max size for each data block
+        - ['mode']:      access mode for the new file
+    """
+    debug = False
+
+    def setup(self, args):
+        self.workdir = args['workdir']
+        self.filename = args['slavedest']
+        self.reader = args['reader']
+        self.bytes_remaining = args['maxsize']
+        self.blocksize = args['blocksize']
+        self.mode = args['mode']
+        self.stderr = None
+        self.rc = 0
+
+    def start(self):
+	if self.debug:
+            log.msg('SlaveFileDownloadCommand starting')
+
+        # Open file
+        self.path = os.path.join(self.builder.basedir,
+                                 self.workdir,
+                                 os.path.expanduser(self.filename))
+        try:
+            self.fp = open(self.path, 'w')
+            if self.debug:
+                log.msg('Opened %r for download' % self.path)
+            if self.mode is not None:
+                # note: there is a brief window during which the new file
+                # will have the buildslave's default (umask) mode before we
+                # set the new one. Don't use this mode= feature to keep files
+                # private: use the buildslave's umask for that instead. (it
+                # is possible to call os.umask() before and after the open()
+                # call, but cleaning up from exceptions properly is more of a
+                # nuisance that way).
+                os.chmod(self.path, self.mode)
+        except IOError:
+            # TODO: this still needs cleanup
+            self.fp = None
+            self.stderr = 'Cannot open file %r for download' % self.path
+            self.rc = 1
+            if self.debug:
+                log.msg('Cannot open file %r for download' % self.path)
+
+        d = defer.Deferred()
+        d.addCallback(self._readBlock)
+        d.addBoth(self.finished)
+        reactor.callLater(0, d.callback, None)
+        return d
+
+    def _readBlock(self, res):
+        """
+        Read a block of data from the remote reader
+        """
+        if self.interrupted or self.fp is None:
+            if self.debug:
+                log.msg('SlaveFileDownloadCommand._readBlock(): end')
+            d = self.reader.callRemote('close')
+            return d
+
+        length = self.blocksize
+        if self.bytes_remaining is not None and length > self.bytes_remaining:
+            length = self.bytes_remaining
+
+        if length <= 0:
+            if self.stderr is None:
+                self.stderr = 'Maximum filesize reached, truncating file %r' \
+                                % self.path
+                self.rc = 1
+            d = self.reader.callRemote('close')
+        else:
+            d = self.reader.callRemote('read', length)
+            d.addCallback(self._writeData)
+        return d
+
+    def _writeData(self, data):
+        if self.debug:
+            log.msg('SlaveFileDownloadCommand._readBlock(): readlen=%d' %
+                    len(data))
+        if len(data) == 0:
+            d = self.reader.callRemote('close')
+            return d
+
+        if self.bytes_remaining is not None:
+            self.bytes_remaining = self.bytes_remaining - len(data)
+            assert self.bytes_remaining >= 0
+        self.fp.write(data)
+        d = self._readBlock(None) # setup call back for next block (or finish)
+        return d
+
+    def interrupt(self):
+        if self.debug:
+            log.msg('interrupted')
+        if self.interrupted:
+            return
+        if self.stderr is None:
+            self.stderr = 'Download of %r interrupted' % self.path
+            self.rc = 1
+        self.interrupted = True
+        # now we wait for the next read request to return. _readBlock will
+        # abandon the file when it sees self.interrupted set.
+
+    def finished(self, res):
+        if self.fp is not None:
+            self.fp.close()
+
+        if self.debug:
+            log.msg('finished: stderr=%r, rc=%r' % (self.stderr, self.rc))
+        if self.stderr is None:
+            self.sendStatus({'rc': self.rc})
+        else:
+            self.sendStatus({'stderr': self.stderr, 'rc': self.rc})
+        return res
+
+registerSlaveCommand("downloadFile", SlaveFileDownloadCommand, command_version)
+
+
+
 class SlaveShellCommand(Command):
     """This is a Command which runs a shell command. The args dict contains
     the following keys:
@@ -739,7 +981,7 @@ registerSlaveCommand("shell", SlaveShellCommand, command_version)
 class DummyCommand(Command):
     """
     I am a dummy no-op command that by default takes 5 seconds to complete.
-    See L{buildbot.process.step.RemoteDummy}
+    See L{buildbot.steps.dummy.RemoteDummy}
     """
     
     def start(self):
@@ -773,6 +1015,56 @@ class DummyCommand(Command):
 registerSlaveCommand("dummy", DummyCommand, command_version)
 
 
+# this maps handle names to a callable. When the WaitCommand starts, this
+# callable is invoked with no arguments. It should return a Deferred. When
+# that Deferred fires, our WaitCommand will finish.
+waitCommandRegistry = {}
+
+class WaitCommand(Command):
+    """
+    I am a dummy command used by the buildbot unit test suite. I want for the
+    unit test to tell us to finish. See L{buildbot.steps.dummy.Wait}
+    """
+    
+    def start(self):
+        self.d = defer.Deferred()
+        log.msg("  starting wait command [%s]" % self.stepId)
+        handle = self.args['handle']
+        cb = waitCommandRegistry[handle]
+        del waitCommandRegistry[handle]
+        def _called():
+            log.msg(" wait-%s starting" % (handle,))
+            d = cb()
+            def _done(res):
+                log.msg(" wait-%s finishing: %s" % (handle, res))
+                return res
+            d.addBoth(_done)
+            d.addCallbacks(self.finished, self.failed)
+        reactor.callLater(0, _called)
+        return self.d
+
+    def interrupt(self):
+        log.msg("  wait command interrupted")
+        if self.interrupted:
+            return
+        self.interrupted = True
+        self.finished("interrupted")
+
+    def finished(self, res):
+        log.msg("  wait command finished [%s]" % self.stepId)
+        if self.interrupted:
+            self.sendStatus({'rc': 2})
+        else:
+            self.sendStatus({'rc': 0})
+        self.d.callback(0)
+    def failed(self, why):
+        log.msg("  wait command failed [%s]" % self.stepId)
+        self.sendStatus({'rc': 1})
+        self.d.callback(0)
+
+registerSlaveCommand("dummy.wait", WaitCommand, command_version)
+
+
 class SourceBase(Command):
     """Abstract base class for Version Control System operations (checkout
     and update). This class extracts the following arguments from the
@@ -1300,6 +1592,112 @@ class Darcs(SourceBase):
 
 registerSlaveCommand("darcs", Darcs, command_version)
 
+class Monotone(SourceBase):
+    """Monotone-specific VC operation.  In addition to the arguments handled
+    by SourceBase, this command reads the following keys:
+
+    ['server_addr'] (required): the address of the server to pull from
+    ['branch'] (required): the branch the revision is on
+    ['db_path'] (required): the local database path to use
+    ['revision'] (required): the revision to check out
+    ['monotone']: (required): path to monotone executable
+    """
+
+    header = "monotone operation"
+
+    def setup(self, args):
+        SourceBase.setup(self, args)
+        self.server_addr = args["server_addr"]
+        self.branch = args["branch"]
+        self.db_path = args["db_path"]
+        self.revision = args["revision"]
+        self.monotone = args["monotone"]
+        self._made_fulls = False
+        self._pull_timeout = args["timeout"]
+
+    def _makefulls(self):
+        if not self._made_fulls:
+            basedir = self.builder.basedir
+            self.full_db_path = os.path.join(basedir, self.db_path)
+            self.full_srcdir = os.path.join(basedir, self.srcdir)
+            self._made_fulls = True
+
+    def sourcedirIsUpdateable(self):
+        self._makefulls()
+        if os.path.exists(os.path.join(self.full_srcdir,
+                                       ".buildbot_patched")):
+            return False
+        return (os.path.isfile(self.full_db_path)
+                and os.path.isdir(os.path.join(self.full_srcdir, "MT")))
+
+    def doVCUpdate(self):
+        return self._withFreshDb(self._doUpdate)
+
+    def _doUpdate(self):
+        # update: possible for mode in ('copy', 'update')
+        command = [self.monotone, "update",
+                   "-r", self.revision,
+                   "-b", self.branch]
+        c = ShellCommand(self.builder, command, self.full_srcdir,
+                         sendRC=False, timeout=self.timeout)
+        self.command = c
+        return c.start()
+
+    def doVCFull(self):
+        return self._withFreshDb(self._doFull)
+
+    def _doFull(self):
+        command = [self.monotone, "--db=" + self.full_db_path,
+                   "checkout",
+                   "-r", self.revision,
+                   "-b", self.branch,
+                   self.full_srcdir]
+        c = ShellCommand(self.builder, command, self.builder.basedir,
+                         sendRC=False, timeout=self.timeout)
+        self.command = c
+        return c.start()
+
+    def _withFreshDb(self, callback):
+        self._makefulls()
+        # first ensure the db exists and is usable
+        if os.path.isfile(self.full_db_path):
+            # already exists, so run 'db migrate' in case monotone has been
+            # upgraded under us
+            command = [self.monotone, "db", "migrate",
+                       "--db=" + self.full_db_path]
+        else:
+            # We'll be doing an initial pull, so up the timeout to 3 hours to
+            # make sure it will have time to complete.
+            self._pull_timeout = max(self._pull_timeout, 3 * 60 * 60)
+            self.sendStatus({"header": "creating database %s\n"
+                                       % (self.full_db_path,)})
+            command = [self.monotone, "db", "init",
+                       "--db=" + self.full_db_path]
+        c = ShellCommand(self.builder, command, self.builder.basedir,
+                         sendRC=False, timeout=self.timeout)
+        self.command = c
+        d = c.start()
+        d.addCallback(self._abandonOnFailure)
+        d.addCallback(self._didDbInit)
+        d.addCallback(self._didPull, callback)
+        return d
+
+    def _didDbInit(self, res):
+        command = [self.monotone, "--db=" + self.full_db_path,
+                   "pull", "--ticker=dot", self.server_addr, self.branch]
+        c = ShellCommand(self.builder, command, self.builder.basedir,
+                         sendRC=False, timeout=self._pull_timeout)
+        self.sendStatus({"header": "pulling %s from %s\n"
+                                   % (self.branch, self.server_addr)})
+        self.command = c
+        return c.start()
+
+    def _didPull(self, res, callback):
+        return callback()
+
+registerSlaveCommand("monotone", Monotone, command_version)
+
+
 class Git(SourceBase):
     """Git specific VC operation. In addition to the arguments
     handled by SourceBase, this command reads the following keys:
