@@ -7,11 +7,16 @@ from twisted.persisted import styles
 from twisted.internet import reactor, defer
 from twisted.protocols import basic
 
-import time, os, os.path, shutil, sys, re, urllib
+import os, shutil, sys, re, urllib
 try:
-    import cPickle as pickle
+    import cPickle
+    pickle = cPickle
 except ImportError:
     import pickle
+try:
+    from cStringIO import StringIO
+except ImportError:
+    from StringIO import StringIO
 
 # sibling imports
 from buildbot import interfaces, util, sourcestamp
@@ -333,6 +338,16 @@ class LogFile:
             else:
                 yield leftover
 
+    def readlines(self, channel=STDOUT):
+        """Return an iterator that produces newline-terminated lines,
+        excluding header chunks."""
+        # TODO: make this memory-efficient, by turning it into a generator
+        # that retrieves chunks as necessary, like a pull-driven version of
+        # twisted.protocols.basic.LineReceiver
+        alltext = "".join(self.getChunks([channel], onlyText=True))
+        io = StringIO(alltext)
+        return io.readlines()
+
     def subscribe(self, receiver, catchup):
         if self.finished:
             return
@@ -410,6 +425,7 @@ class LogFile:
         self.finishedWatchers = []
         for w in watchers:
             w.callback(self)
+        self.watchers = []
 
     # persistence stuff
     def __getstate__(self):
@@ -1493,7 +1509,8 @@ class BuilderStatus(styles.Versioned):
 
         # remember the oldest-to-earliest flow here. "next" means earlier.
 
-        # TODO: interleave build steps and self.events by timestamp
+        # TODO: interleave build steps and self.events by timestamp.
+        # TODO: um, I think we're already doing that.
 
         eventIndex = -1
         e = self.getEvent(eventIndex)
@@ -1724,6 +1741,13 @@ class SlaveStatus:
     def isConnected(self):
         return self.connected
 
+    def setAdmin(self, admin):
+        self.admin = admin
+    def setHost(self, host):
+        self.host = host
+    def setConnected(self, isConnected):
+        self.connected = isConnected
+
 class Status:
     """
     I represent the status of the buildmaster.
