@@ -3,7 +3,6 @@
 from __future__ import generators
 
 from twisted.python import log, components
-from twisted.python.util import sibpath
 import urllib, re
 
 from twisted.internet import defer, reactor
@@ -145,14 +144,18 @@ class Box:
 class HtmlResource(Resource):
     css = None
     contentType = "text/html; charset=UTF-8"
+    title = "Dummy"
+
     def render(self, request):
         data = self.content(request)
+        if isinstance(data, unicode):
+            data = data.encode("utf-8")
         request.setHeader("content-type", self.contentType)
         if request.method == "HEAD":
             request.setHeader("content-length", len(data))
             return ''
         return data
-    title = "Dummy"
+
     def content(self, request):
         data = ('<!DOCTYPE html PUBLIC'
                 ' "-//W3C//DTD XHTML 1.0 Transitional//EN"\n'
@@ -173,6 +176,7 @@ class HtmlResource(Resource):
         data += self.body(request)
         data += "</body></html>\n"
         return data
+
     def body(self, request):
         return "Dummy\n"
 
@@ -325,10 +329,10 @@ class StatusResourceBuild(HtmlResource):
                                                                  projectName)
         # the color in the following line gives python-mode trouble
         data += ("<h1>Build <a href=\"%s\">%s</a>:#%d</h1>\n"
-                 "<h2>Reason:</h2>\n%s\n"
                  % (self.status.getURLForThing(b.getBuilder()),
-                    b.getBuilder().getName(), b.getNumber(),
-                    html.escape(b.getReason())))
+                    b.getBuilder().getName(), b.getNumber()))
+        data += "<h2>Buildslave:</h2>\n %s\n" % html.escape(b.getSlavename())
+        data += "<h2>Reason:</h2>\n%s\n" % html.escape(b.getReason())
 
         branch, revision, patch = b.getSourceStamp()
         data += "<h2>SourceStamp:</h2>\n"
@@ -346,7 +350,6 @@ class StatusResourceBuild(HtmlResource):
             data += "  <li>build of most recent revision</li>\n"
         data += " </ul>\n"
         if b.isFinished():
-            data += "<h4>Buildslave: %s</h4>\n" % html.escape(b.getSlavename())
             data += "<h2>Results:</h2>\n"
             data += " ".join(b.getText()) + "\n"
             if b.getTestResults():
