@@ -13,7 +13,6 @@ except ImportError:
 import urllib
 
 from twisted.internet import defer
-from twisted.application import service
 try:
     from twisted.mail.smtp import sendmail # Twisted-2.0
 except ImportError:
@@ -95,7 +94,7 @@ class MailNotifier(base.StatusReceiverMultiService):
         @type  subject: string
         @param subject: a string to be used as the subject line of the message.
                         %(builder)s will be replaced with the name of the
-                        %builder which provoked the message.
+                        builder which provoked the message.
 
         @type  mode: string (defaults to all)
         @param mode: one of:
@@ -240,6 +239,7 @@ class MailNotifier(base.StatusReceiverMultiService):
         if url:
             text += "Buildbot URL: %s\n\n" % urllib.quote(url, '/:')
 
+        text += "Buildslave for this Build: %s\n\n" % build.getSlavename()
         text += "Build Reason: %s\n" % build.getReason()
 
         patch = None
