@@ -9,7 +9,7 @@ from buildbot.twcompat import maybeWait
 from buildbot.slave import bot
 from buildbot.process.builder import Builder
 from buildbot.process.base import BuildRequest, Build
-from buildbot.process.step import BuildStep
+from buildbot.process.buildstep import BuildStep
 from buildbot.sourcestamp import SourceStamp
 from buildbot.status import builder
 
@@ -229,6 +229,10 @@ def setupBuildStepStatus(basedir):
     s3.stepStarted()
     return s3
 
+def fake_slaveVersion(command, oldversion=None):
+    from buildbot.slave.registry import commandRegistry
+    return commandRegistry[command]
+
 def makeBuildStep(basedir, step_class=BuildStep, **kwargs):
     bss = setupBuildStepStatus(basedir)
 
@@ -242,6 +246,7 @@ def makeBuildStep(basedir, step_class=BuildStep, **kwargs):
     s = step_class(build=b, **kwargs)
     s.setStepStatus(bss)
     b.setupStatus(bss.getBuild())
+    s.slaveVersion = fake_slaveVersion
     return s
 
 
@@ -319,3 +324,93 @@ class SlaveCommandTestBase(SignalMixin):
             if "stderr" in u:
                 print u["stderr"]
 
+# ----------------------------------------
+
+class LocalWrapper:
+    # r = pb.Referenceable()
+    # w = LocalWrapper(r)
+    # now you can do things like w.callRemote()
+    def __init__(self, target):
+        self.target = target
+
+    def callRemote(self, name, *args, **kwargs):
+        d = defer.maybeDeferred(self._callRemote, name, *args, **kwargs)
+        return d
+
+    def _callRemote(self, name, *args, **kwargs):
+        method = getattr(self.target, "remote_"+name)
+        return method(*args, **kwargs)
+
+    def notifyOnDisconnect(self, observer):
+        pass
+    def dontNotifyOnDisconnect(self, observer):
+        pass
+
+
+class LocalSlaveBuilder(bot.SlaveBuilder):
+    """I am object that behaves like a pb.RemoteReference, but in fact I
+    invoke methods locally."""
+    _arg_filter = None
+
+    def setArgFilter(self, filter):
+        self._arg_filter = filter
+
+    def remote_startCommand(self, stepref, stepId, command, args):
+        if self._arg_filter:
+            args = self._arg_filter(args)
+        # stepref should be a RemoteReference to the RemoteCommand
+        return bot.SlaveBuilder.remote_startCommand(self,
+                                                    LocalWrapper(stepref),
+                                                    stepId, command, args)
+
+class StepTester:
+    """Utility class to exercise BuildSteps and RemoteCommands, without
+    really using a Build or a Bot. No networks are used.
+
+    Use this as follows::
+
+    class MyTest(StepTester, unittest.TestCase):
+        def testOne(self):
+            self.slavebase = 'testOne.slave'
+            self.masterbase = 'testOne.master'
+            sb = self.makeSlaveBuilder()
+            step = self.makeStep(stepclass, **kwargs)
+            d = self.runStep(step)
+            d.addCallback(_checkResults)
+            return d
+    """
+
+    #slavebase = "slavebase"
+    slavebuilderbase = "slavebuilderbase"
+    #masterbase = "masterbase"
+
+    def makeSlaveBuilder(self):
+        os.mkdir(self.slavebase)
+        os.mkdir(os.path.join(self.slavebase, self.slavebuilderbase))
+        b = bot.Bot(self.slavebase, False)
+        b.startService()
+        sb = LocalSlaveBuilder("slavebuildername", False)
+        sb.setArgFilter(self.filterArgs)
+        sb.usePTY = False
+        sb.setServiceParent(b)
+        sb.setBuilddir(self.slavebuilderbase)
+        self.remote = LocalWrapper(sb)
+        return sb
+
+    workdir = "build"
+    def makeStep(self, factory, **kwargs):
+        if not kwargs.has_key("workdir"):
+            kwargs['workdir'] = self.workdir
+        step = makeBuildStep(self.masterbase, factory, **kwargs)
+        return step
+
+    def runStep(self, step):
+        d = defer.maybeDeferred(step.startStep, self.remote)
+        return d
+
+    def wrap(self, target):
+        return LocalWrapper(target)
+
+    def filterArgs(self, args):
+        # this can be overridden
+        return args
