@@ -2,13 +2,10 @@
 
 from twisted.trial import unittest
 from twisted.internet import reactor, defer
-from twisted.python import log
-import sys, os, os.path, shutil, time, errno
-#log.startLogging(sys.stderr)
+import os
 
 from buildbot import master, interfaces
 from buildbot.sourcestamp import SourceStamp
-from buildbot.slave import bot
 from buildbot.changes import changes
 from buildbot.status import builder
 from buildbot.process.base import BuildRequest
@@ -17,14 +14,15 @@ from buildbot.twcompat import maybeWait
 from buildbot.test.runutils import RunMixin, rmtree
 
 config_base = """
-from buildbot.process import factory, step
+from buildbot.process import factory
+from buildbot.steps import dummy
 s = factory.s
 
 f1 = factory.QuickBuildFactory('fakerep', 'cvsmodule', configure=None)
 
 f2 = factory.BuildFactory([
-    s(step.Dummy, timeout=1),
-    s(step.RemoteDummy, timeout=2),
+    s(dummy.Dummy, timeout=1),
+    s(dummy.RemoteDummy, timeout=2),
     ])
 
 BuildmasterConfig = c = {}
@@ -176,19 +174,16 @@ class Disconnect(RunMixin, unittest.TestCase):
 
         self.failUnlessEqual(bs.getResults(), builder.FAILURE)
 
-
-    def testIdle1(self):
-        # disconnect the slave before the build starts
-        d = self.shutdownAllSlaves() # dies before it gets started
-        d.addCallback(self._testIdle1_1)
+    def submitBuild(self):
+        ss = SourceStamp()
+        br = BuildRequest("forced build", ss, "dummy")
+        self.control.getBuilder("dummy").requestBuild(br)
+        d = defer.Deferred()
+        def _started(bc):
+            br.unsubscribe(_started)
+            d.callback(bc)
+        br.subscribe(_started)
         return d
-    def _testIdle1_1(self, res):
-        # trying to force a build now will cause an error. Regular builds
-        # just wait for the slave to re-appear, but forced builds that
-        # cannot be run right away trigger NoSlaveErrors
-        fb = self.control.getBuilder("dummy").forceBuild
-        self.failUnlessRaises(interfaces.NoSlaveError,
-                              fb, None, "forced build")
 
     def testIdle2(self):
         # now suppose the slave goes missing
@@ -222,7 +217,7 @@ class Disconnect(RunMixin, unittest.TestCase):
         # least 3 seconds to complete, and this batch of commands must
         # complete within that time.
         #
-        d = self.control.getBuilder("dummy").forceBuild(None, "forced build")
+        d = self.submitBuild()
         d.addCallback(self._testBuild1_1)
         return maybeWait(d)
 
@@ -251,7 +246,7 @@ class Disconnect(RunMixin, unittest.TestCase):
 
     def testBuild2(self):
         # this next sequence is timing-dependent
-        d = self.control.getBuilder("dummy").forceBuild(None, "forced build")
+        d = self.submitBuild()
         d.addCallback(self._testBuild1_1)
         return maybeWait(d, 30)
     testBuild2.timeout = 30
@@ -281,7 +276,7 @@ class Disconnect(RunMixin, unittest.TestCase):
 
     def testBuild3(self):
         # this next sequence is timing-dependent
-        d = self.control.getBuilder("dummy").forceBuild(None, "forced build")
+        d = self.submitBuild()
         d.addCallback(self._testBuild3_1)
         return maybeWait(d, 30)
     testBuild3.timeout = 30
@@ -308,7 +303,7 @@ class Disconnect(RunMixin, unittest.TestCase):
 
     def testBuild4(self):
         # this next sequence is timing-dependent
-        d = self.control.getBuilder("dummy").forceBuild(None, "forced build")
+        d = self.submitBuild()
         d.addCallback(self._testBuild4_1)
         return maybeWait(d, 30)
     testBuild4.timeout = 30
@@ -333,7 +328,7 @@ class Disconnect(RunMixin, unittest.TestCase):
 
     def testInterrupt(self):
         # this next sequence is timing-dependent
-        d = self.control.getBuilder("dummy").forceBuild(None, "forced build")
+        d = self.submitBuild()
         d.addCallback(self._testInterrupt_1)
         return maybeWait(d, 30)
     testInterrupt.timeout = 30
@@ -499,10 +494,10 @@ class Basedir(RunMixin, unittest.TestCase):
 
     def _testChangeBuilddir_2(self, res):
         bot = self.bot
-        # this causes the builder to be replaced
-        self.failIfIdentical(self.builder, bot.builders.get("dummy"))
+        # this does NOT cause the builder to be replaced
         builder = bot.builders.get("dummy")
         self.failUnless(builder)
+        self.failUnlessIdentical(self.builder, builder)
         # the basedir should be updated
         self.failUnlessEqual(builder.builddir, "dummy2")
         self.failUnlessEqual(builder.basedir,
