@@ -10,10 +10,12 @@ from buildbot.sourcestamp import SourceStamp
 from buildbot.process.base import BuildRequest
 from buildbot.twcompat import implements, providedBy, maybeWait
 from buildbot.status import builder, base
+
+mail = None
 try:
     from buildbot.status import mail
 except ImportError:
-    mail = None
+    pass
 from buildbot.status import progress, client # NEEDS COVERAGE
 from buildbot.test.runutils import RunMixin
 
@@ -486,6 +488,32 @@ class Log(unittest.TestCase):
         l.finish()
         self.failUnlessEqual(l.getText(), 160*"a")
 
+    def testReadlines(self):
+        l = MyLog(self.basedir, "chunks")
+        l.addHeader("HEADER\n") # should be ignored
+        l.addStdout("Some text\n")
+        l.addStdout("Some More Text\nAnd Some More\n")
+        l.addStderr("Some Stderr\n")
+        l.addStdout("Last line\n")
+        l.finish()
+        alllines = list(l.readlines())
+        self.failUnlessEqual(len(alllines), 4)
+        self.failUnlessEqual(alllines[0], "Some text\n")
+        self.failUnlessEqual(alllines[2], "And Some More\n")
+        self.failUnlessEqual(alllines[3], "Last line\n")
+        stderr = list(l.readlines(interfaces.LOG_CHANNEL_STDERR))
+        self.failUnlessEqual(len(stderr), 1)
+        self.failUnlessEqual(stderr[0], "Some Stderr\n")
+        lines = l.readlines()
+        if False: # TODO: l.readlines() is not yet an iterator
+            # verify that it really is an iterator
+            line0 = lines.next()
+            self.failUnlessEqual(line0, "Some text\n")
+            line1 = lines.next()
+            line2 = lines.next()
+            self.failUnlessEqual(line2, "And Some More\n")
+
+
     def testChunks(self):
         l = MyLog(self.basedir, "chunks")
         c1 = l.getChunks()
@@ -697,14 +725,15 @@ class Log(unittest.TestCase):
     testLargeSummary.timeout = 5
 
 config_base = """
-from buildbot.process import factory, step
+from buildbot.process import factory
+from buildbot.steps import dummy
 s = factory.s
 
 f1 = factory.QuickBuildFactory('fakerep', 'cvsmodule', configure=None)
 
 f2 = factory.BuildFactory([
-    s(step.Dummy, timeout=1),
-    s(step.RemoteDummy, timeout=2),
+    s(dummy.Dummy, timeout=1),
+    s(dummy.RemoteDummy, timeout=2),
     ])
 
 BuildmasterConfig = c = {}
