@@ -14,30 +14,28 @@
 # todo: test batched updates, by invoking remote_update(updates) instead of
 # statusUpdate(update). Also involves interrupted builds.
 
-import os, sys, time
+import os, time
 
 from twisted.trial import unittest
-from twisted.internet import reactor
-from twisted.internet.defer import Deferred
+from twisted.internet import reactor, defer
 
 from buildbot.sourcestamp import SourceStamp
-from buildbot.process import step, base, factory
-from buildbot.process.step import ShellCommand #, ShellCommands
+from buildbot.process import buildstep, base, factory
+from buildbot.steps import shell, source, python
 from buildbot.status import builder
-from buildbot.test.runutils import RunMixin, rmtree, setupBuildStepStatus
-from buildbot.test.runutils import makeBuildStep
+from buildbot.status.builder import SUCCESS, FAILURE
+from buildbot.test.runutils import RunMixin, rmtree
+from buildbot.test.runutils import makeBuildStep, StepTester
 from buildbot.twcompat import maybeWait
-from buildbot.slave import commands
+from buildbot.slave import commands, registry
 
-from twisted.python import log
-#log.startLogging(sys.stdout)
 
-class MyShellCommand(ShellCommand):
+class MyShellCommand(shell.ShellCommand):
     started = False
     def runCommand(self, c):
         self.started = True
         self.rc = c
-        return ShellCommand.runCommand(self, c)
+        return shell.ShellCommand.runCommand(self, c)
 
 class FakeBuild:
     pass
@@ -59,7 +57,7 @@ class FakeRemote:
 ##         if self.callRemoteNotifier:
 ##             reactor.callLater(0, self.callRemoteNotifier, event)
         self.remoteCalls += 1
-        self.deferred = Deferred()
+        self.deferred = defer.Deferred()
         return self.deferred
     def notifyOnDisconnect(self, callback):
         pass
@@ -100,7 +98,7 @@ class BuildStep(unittest.TestCase):
         cmd = "argle bargle"
         dir = "murkle"
         expectedEvents = []
-        step.RemoteCommand.commandCounter[0] = 3
+        buildstep.RemoteCommand.commandCounter[0] = 3
         c = MyShellCommand(workdir=dir, command=cmd, build=self.build,
                            timeout=10)
         self.assertEqual(self.remote.events, expectedEvents)
@@ -161,7 +159,7 @@ class BuildStep(unittest.TestCase):
         self.assertEqual(self.results, 0)
 
 
-class MyObserver(step.LogObserver):
+class MyObserver(buildstep.LogObserver):
     out = ""
     def outReceived(self, data):
         self.out = self.out + data
@@ -169,13 +167,13 @@ class MyObserver(step.LogObserver):
 class Steps(unittest.TestCase):
     def testMultipleStepInstances(self):
         steps = [
-            (step.CVS, {'cvsroot': "root", 'cvsmodule': "module"}),
-            (step.Configure, {'command': "./configure"}),
-            (step.Compile, {'command': "make"}),
-            (step.Compile, {'command': "make more"}),
-            (step.Compile, {'command': "make evenmore"}),
-            (step.Test, {'command': "make test"}),
-            (step.Test, {'command': "make testharder"}),
+            (source.CVS, {'cvsroot': "root", 'cvsmodule': "module"}),
+            (shell.Configure, {'command': "./configure"}),
+            (shell.Compile, {'command': "make"}),
+            (shell.Compile, {'command': "make more"}),
+            (shell.Compile, {'command': "make evenmore"}),
+            (shell.Test, {'command': "make test"}),
+            (shell.Test, {'command': "make testharder"}),
             ]
         f = factory.ConfigurableBuildFactory(steps)
         req = base.BuildRequest("reason", SourceStamp())
@@ -263,7 +261,7 @@ class Steps(unittest.TestCase):
 
     def test_description(self):
         s = makeBuildStep("test_steps.Steps.test_description.1",
-                          step_class=ShellCommand,
+                          step_class=shell.ShellCommand,
                           workdir="dummy",
                           description=["list", "of", "strings"],
                           descriptionDone=["another", "list"])
@@ -271,14 +269,14 @@ class Steps(unittest.TestCase):
         self.failUnlessEqual(s.descriptionDone, ["another", "list"])
 
         s = makeBuildStep("test_steps.Steps.test_description.2",
-                          step_class=ShellCommand,
+                          step_class=shell.ShellCommand,
                           workdir="dummy",
                           description="single string",
                           descriptionDone="another string")
         self.failUnlessEqual(s.description, ["single string"])
         self.failUnlessEqual(s.descriptionDone, ["another string"])
 
-class VersionCheckingStep(step.BuildStep):
+class VersionCheckingStep(buildstep.BuildStep):
     def start(self):
         # give our test a chance to run. It is non-trivial for a buildstep to
         # claw its way back out to the test case which is currently running.
@@ -286,10 +284,10 @@ class VersionCheckingStep(step.BuildStep):
         checker = master._checker
         checker(self)
         # then complete
-        self.finished(step.SUCCESS)
+        self.finished(buildstep.SUCCESS)
 
 version_config = """
-from buildbot.process import factory, step
+from buildbot.process import factory
 from buildbot.test.test_steps import VersionCheckingStep
 BuildmasterConfig = c = {}
 f1 = factory.BuildFactory([
@@ -342,3 +340,162 @@ class SlaveVersion(RunMixin, unittest.TestCase):
         d = self.doBuild("quick")
         return maybeWait(d)
 
+
+class ReorgCompatibility(unittest.TestCase):
+    def testCompat(self):
+        from buildbot.process.step import LogObserver, LogLineObserver
+        from buildbot.process.step import RemoteShellCommand
+        from buildbot.process.step import BuildStep, LoggingBuildStep
+        from buildbot.process.step import ShellCommand, WithProperties
+        from buildbot.process.step import TreeSize
+        from buildbot.process.step import Configure
+        from buildbot.process.step import Compile
+        from buildbot.process.step import Test
+        from buildbot.process.step import CVS
+        from buildbot.process.step import SVN
+        from buildbot.process.step import Darcs
+        from buildbot.process.step import Git
+        from buildbot.process.step import Arch
+        from buildbot.process.step import Bazaar
+        from buildbot.process.step import Mercurial
+        from buildbot.process.step import P4
+        from buildbot.process.step import P4Sync
+        from buildbot.process.step import Dummy
+        from buildbot.process.step import FailingDummy
+        from buildbot.process.step import RemoteDummy
+
+        # now trick pyflakes into thinking we care
+        unused = [LogObserver, LogLineObserver, RemoteShellCommand,
+                  BuildStep, LoggingBuildStep, ShellCommand, WithProperties,
+                  TreeSize, Configure, Compile, Test, CVS, SVN, Darcs,
+                  Git, Arch, Bazaar, Mercurial, P4, P4Sync,
+                  Dummy, FailingDummy, RemoteDummy]
+
+
+class _SimpleBuildStep(buildstep.BuildStep):
+    def start(self):
+        args = {"arg1": "value"}
+        cmd = buildstep.RemoteCommand("simple", args)
+        d = self.runCommand(cmd)
+        d.addCallback(lambda res: self.finished(SUCCESS))
+
+class _SimpleCommand(commands.Command):
+    def start(self):
+        self.builder.flag = True
+        self.builder.flag_args = self.args
+        return defer.succeed(None)
+
+class CheckStepTester(StepTester, unittest.TestCase):
+    def testSimple(self):
+        self.slavebase = "testSimple.slave"
+        self.masterbase = "testSimple.master"
+        sb = self.makeSlaveBuilder()
+        sb.flag = False
+        registry.registerSlaveCommand("simple", _SimpleCommand, "1")
+        step = self.makeStep(_SimpleBuildStep)
+        d = self.runStep(step)
+        def _checkSimple(results):
+            self.failUnless(sb.flag)
+            self.failUnlessEqual(sb.flag_args, {"arg1": "value"})
+        d.addCallback(_checkSimple)
+        return maybeWait(d)
+
+class Python(StepTester, unittest.TestCase):
+    def testPyFlakes1(self):
+        self.masterbase = "Python.testPyFlakes1"
+        step = self.makeStep(python.PyFlakes)
+        output = \
+"""pyflakes buildbot
+buildbot/changes/freshcvsmail.py:5: 'FCMaildirSource' imported but unused
+buildbot/clients/debug.py:9: redefinition of unused 'gtk' from line 9
+buildbot/clients/debug.py:9: 'gnome' imported but unused
+buildbot/scripts/runner.py:323: redefinition of unused 'run' from line 321
+buildbot/scripts/runner.py:325: redefinition of unused 'run' from line 323
+buildbot/scripts/imaginary.py:12: undefined name 'size'
+buildbot/scripts/imaginary.py:18: 'from buildbot import *' used; unable to detect undefined names
+"""
+        log = step.addLog("stdio")
+        log.addStdout(output)
+        log.finish()
+        step.createSummary(log)
+        desc = step.descriptionDone
+        self.failUnless("unused=2" in desc)
+        self.failUnless("undefined=1" in desc)
+        self.failUnless("redefs=3" in desc)
+        self.failUnless("import*=1" in desc)
+        self.failIf("misc=" in desc)
+
+        self.failUnlessEqual(step.getProperty("pyflakes-unused"), 2)
+        self.failUnlessEqual(step.getProperty("pyflakes-undefined"), 1)
+        self.failUnlessEqual(step.getProperty("pyflakes-redefs"), 3)
+        self.failUnlessEqual(step.getProperty("pyflakes-import*"), 1)
+        self.failUnlessEqual(step.getProperty("pyflakes-misc"), 0)
+        self.failUnlessEqual(step.getProperty("pyflakes-total"), 7)
+
+        logs = {}
+        for log in step.step_status.getLogs():
+            logs[log.getName()] = log
+
+        for name in ["unused", "undefined", "redefs", "import*"]:
+            self.failUnless(name in logs)
+        self.failIf("misc" in logs)
+        lines = logs["unused"].readlines()
+        self.failUnlessEqual(len(lines), 2)
+        self.failUnlessEqual(lines[0], "buildbot/changes/freshcvsmail.py:5: 'FCMaildirSource' imported but unused\n")
+
+        cmd = buildstep.RemoteCommand(None, {})
+        cmd.rc = 0
+        results = step.evaluateCommand(cmd)
+        self.failUnlessEqual(results, FAILURE) # because of the 'undefined'
+
+    def testPyFlakes2(self):
+        self.masterbase = "Python.testPyFlakes2"
+        step = self.makeStep(python.PyFlakes)
+        output = \
+"""pyflakes buildbot
+some more text here that should be ignored
+buildbot/changes/freshcvsmail.py:5: 'FCMaildirSource' imported but unused
+buildbot/clients/debug.py:9: redefinition of unused 'gtk' from line 9
+buildbot/clients/debug.py:9: 'gnome' imported but unused
+buildbot/scripts/runner.py:323: redefinition of unused 'run' from line 321
+buildbot/scripts/runner.py:325: redefinition of unused 'run' from line 323
+buildbot/scripts/imaginary.py:12: undefined name 'size'
+could not compile 'blah/blah.py':3:
+pretend there was an invalid line here
+buildbot/scripts/imaginary.py:18: 'from buildbot import *' used; unable to detect undefined names
+"""
+        log = step.addLog("stdio")
+        log.addStdout(output)
+        log.finish()
+        step.createSummary(log)
+        desc = step.descriptionDone
+        self.failUnless("unused=2" in desc)
+        self.failUnless("undefined=1" in desc)
+        self.failUnless("redefs=3" in desc)
+        self.failUnless("import*=1" in desc)
+        self.failUnless("misc=2" in desc)
+
+
+    def testPyFlakes3(self):
+        self.masterbase = "Python.testPyFlakes3"
+        step = self.makeStep(python.PyFlakes)
+        output = \
+"""buildbot/changes/freshcvsmail.py:5: 'FCMaildirSource' imported but unused
+buildbot/clients/debug.py:9: redefinition of unused 'gtk' from line 9
+buildbot/clients/debug.py:9: 'gnome' imported but unused
+buildbot/scripts/runner.py:323: redefinition of unused 'run' from line 321
+buildbot/scripts/runner.py:325: redefinition of unused 'run' from line 323
+buildbot/scripts/imaginary.py:12: undefined name 'size'
+buildbot/scripts/imaginary.py:18: 'from buildbot import *' used; unable to detect undefined names
+"""
+        log = step.addLog("stdio")
+        log.addStdout(output)
+        log.finish()
+        step.createSummary(log)
+        desc = step.descriptionDone
+        self.failUnless("unused=2" in desc)
+        self.failUnless("undefined=1" in desc)
+        self.failUnless("redefs=3" in desc)
+        self.failUnless("import*=1" in desc)
+        self.failIf("misc" in desc)
+
