@@ -1,8 +1,7 @@
 # -*- test-case-name: buildbot.test.test_web -*-
 
-import sys, os, os.path, time, shutil
-from twisted.python import log, components, util
-#log.startLogging(sys.stderr)
+import os, time, shutil
+from twisted.python import components
 
 from twisted.trial import unittest
 from buildbot.test.runutils import RunMixin
@@ -11,11 +10,12 @@ from twisted.internet import reactor, defer, protocol
 from twisted.internet.interfaces import IReactorUNIX
 from twisted.web import client
 
-from buildbot import master, interfaces, buildset, sourcestamp
+from buildbot import master, interfaces, sourcestamp
 from buildbot.twcompat import providedBy, maybeWait
 from buildbot.status import html, builder
 from buildbot.changes.changes import Change
-from buildbot.process import step, base
+from buildbot.process import base
+from buildbot.process.buildstep import BuildStep
 from buildbot.test.runutils import setupBuildStepStatus
 
 class ConfiguredMaster(master.BuildMaster):
@@ -280,7 +280,8 @@ class WaterfallSteps(unittest.TestCase):
 geturl_config = """
 from buildbot.status import html
 from buildbot.changes import mail
-from buildbot.process import step, factory
+from buildbot.process import factory
+from buildbot.steps import dummy
 from buildbot.scheduler import Scheduler
 from buildbot.changes.base import ChangeSource
 s = factory.s
@@ -298,7 +299,7 @@ c['schedulers'] = [DiscardScheduler('discard', None, 60, ['b1'])]
 c['slavePortnum'] = 0
 c['status'] = [html.Waterfall(http_port=0)]
 
-f = factory.BuildFactory([s(step.RemoteDummy, timeout=1)])
+f = factory.BuildFactory([s(dummy.RemoteDummy, timeout=1)])
 
 c['builders'] = [
     {'name': 'b1', 'slavenames': ['bot1','bot2'],
@@ -414,7 +415,7 @@ BuildmasterConfig = {
         bs.setReason("reason")
         bs.buildStarted(build1)
 
-        step1 = step.BuildStep(build=build1, name="setup")
+        step1 = BuildStep(build=build1, name="setup")
         bss = bs.addStepWithName("setup")
         step1.setStepStatus(bss)
         bss.stepStarted()
